if (!require('rgbif')) install.packages('rgbif', type='source')

# load libraries
library(shiny)
library(rgbif)
library(leaflet)


shinyServer(function(input, output, session) {
  
  # this list carries data that is used by multiple reactive functions
  values <- reactiveValues()
  
  # query GBIF based on user input, remove duplicate records
  GBIFsearch <- reactive({
    if (input$gbifName == '') return(NULL)
    input$goName
    isolate({
      withProgress(message = "Searching GBIF...", {
        results <- occ_search(scientificName = input$gbifName, limit = input$occurrences, 
                              fields = c('name', 'decimalLongitude', 'decimalLatitude', 'basisOfRecord'), 
                              hasCoordinate = TRUE)
        if (results$meta$count != 0) {
          locs <- results$data[!is.na(results$data[,3]),][,c(1,3,4,2)]
          dup <- duplicated(locs)
          locs <- locs[!dup, ]
          names(locs)[2:3] <- c('lon', 'lat')
          values$gbifoccs <- locs
        }
      })
    })
  })
  
  
  # map of GBIF records
  map1 <- leaflet() %>% addTiles() %>% setView(0, 0, zoom = 2)
  output$map1 <- output$map2 <- renderLeaflet(map1)
  
  #   # map of thinned GBIF records
  #   map2 <- leaflet() %>% addTiles() %>% setView(0, 0, zoom = 2)
  #   output$map2 <- renderLeaflet(map2)
  
  observeEvent(input$goName, {
    proxy1 <- leafletProxy("map1")
    proxy2 <- leafletProxy("map2")
    GBIFsearch()
    proxy1 %>% clearShapes()
    proxy2 %>% clearShapes()
    lati <- values$gbifoccs[,3]
    longi <- values$gbifoccs[,2]
    proxy1 %>% fitBounds(min(longi), min(lati), max(longi), max(lati))
    proxy2 %>% fitBounds(min(longi), min(lati), max(longi), max(lati))
        
    proxy1 %>% addCircleMarkers(data = values$gbifoccs, lat = ~lat, lng = ~lon, 
                                layerId = as.numeric(rownames(values$gbifoccs)), 
                                radius = 5, color = 'red', fill = FALSE, weight = 2)
    proxy2 %>% addCircleMarkers(data = values$gbifoccs, lat = ~lat, lng = ~lon, 
                                layerId = as.numeric(rownames(values$gbifoccs)), 
                                radius = 5, color = 'red', fill = FALSE, weight = 2)
  })
    
  # map thinned records when Thin button is pressed
  observeEvent(input$goThin, {
    proxy2 <- leafletProxy("map2")
    thinoccs <- values$gbifoccs[sample(nrow(values$gbifoccs), nrow(values$gbifoccs) - input$pickRand),]
    print(thinoccs)
    lati2 <- thinoccs[,3]
    longi2 <- thinoccs[,2]
    proxy2 %>% clearShapes()
    proxy2 %>% fitBounds(min(longi2), min(lati2), max(longi2), max(lati2))
    proxy2 %>% addCircleMarkers(data = thinoccs, lat = ~lat, lng = ~lon, 
                                layerId = as.numeric(rownames(thinoccs)), 
                                radius = 5, color = 'blue', fill = FALSE, weight = 2)
  })  
})
